;****************************************************
;Jakub Borzdynski 2005
;-------------------------
;Program do obslugi sterownika pieca C.O.
;
;****************************************************
;OPIS WYPROWADZEN
;L1=PB2, L2=PB1 , L3=PB3 , L4=PB0
;JP1=PB1, JP2=PB2, JP3=PB4
;S_dol=PB3, S_gora=PB0
;
;MAPA EEPROM
; 00H -> kroki/st. C
; 01H -> temperatura wlaczenia
;***************************************************
;definicje
;==============
;******************
.INCLUDE "tn15def.inc"    ; Include I/O definitions 
.def temp  =r16 ;zmienna pomocnicza
.def KROKI =r17 ;kroki/stopien C
.def T_WL  =r18 ;przy jakiej temp. sie uakt.
.def BIEG  =r19 ;predkosc
.def DELTA =r20 ;do obliczania zmiany temp.
.def posrednik =r21 ;posredniczy przy wpisywaniu wartosci
.def wait1 =r22 ;generacja opoznienia
.def wait2 =r23 ;generacja opoznienia
.def wait3 =r24 ;generacja opoznienia
.def SILNIK =r25 ;rejestr przesuwny dla silnika
.def pomiar =r26 ;buforuje pomiar z ADC
.def adres =r27 ;adres zapisu/odczytu EEPROM
.def dane =r28 ;EEPROM: dane zapisywane/odczytywane
.equ DEF_temp =10  ;wartosc domyslna temperatury
.equ DEF_kroki =1 ;wartosc domyslna liczby krokow
.equ DEF_bieg =1   ;wartosc domyslna biegu
;####################################################
;####################################################
RESET: ;tu sie zaczyna program po resecie

	;ustaw maska silnika
	ldi SILNIK, 0x01
	
	;odczyt konfiguracji z pamieci EEPROM
	ldi adres, 0 ;odczyt krokow
	rcall ODCZYT_EEPROM
	mov KROKI, dane
	ldi adres, 1 ;odczyt temperatury
	rcall ODCZYT_EEPROM
	mov T_WL, dane
		
	;ustawienie globalnej zmiennej temp
	mov temp,T_WL
	
	;przejscie do petli MAIN
;####################################################
;####################################################
MAIN: ;program glowny
	;ustawienie JP1, JP2, JP3 jako wejscia z pull-up
	ldi posrednik, 0x00
	out DDRB, posrednik  ;jako wejscie
	ldi posrednik, 0xFF
	out PORTB, posrednik ;z pull-up
	
	sbic PINB, 1 ;czy do trybu programowania to krok?
	rjmp RUN
	sbic PINB, 2 ;czy do trybu programowania to krok?
	rjmp RUN
	
	;przejscie do trybu programowania
	
;####################################################
;####################################################
TRYB_PROG: ;petla konfiguracji sterownika

	;czy zresetowac do domyslnych? Oba switche wcisniete
	sbic PINB, 0
	rjmp SETUP_T
	sbic PINB, 3
	rjmp SETUP_T
	
	;przywrocenie wartosci domyslnych
	ldi T_WL, DEF_temp
	ldi KROKI, DEF_kroki
	
	;konfiguracja temperatury
	SETUP_T:
		;------------------------ 
		T_czy_DOL:
			;przycisk S_dol wcisniety?
			sbic PINB,3
			rjmp T_czy_GORA ;jedynka, nie byl wcisniety
			;badanie wartosci i ew. zmniejszenie
			ldi posrednik, 5 ;do porownania (dolny zakres)
			cpse T_WL,posrednik
			rcall T_MINUS ;nie bylo zakresu dolnego - zmniejsz
			
		T_czy_GORA:
			;przycisk S_gora wcisniety?
			sbic PINB,0
			rjmp temp_END ;jedynka, nie byl wcisniety
			;badanie wartosci i ew. zmniejszenie
			ldi posrednik, 50 ;do porownania
			cpse T_WL,posrednik
			rcall T_PLUS ;nie bylo zakresu gornego - zwieksz
		;zakonczono zmiane wartosci KROKow
	temp_END:
		;------------------------
	sbis PINB, 1 ;pomin gdy JP1=1
	rjmp SETUP_T
		
	;koniguracja krokow
	SETUP_K:
		;przycisk S_dol wcisniety?
		sbic PINB,3
		rjmp czy_GORA ;jedynka, nie byl wcisniety
		;badanie wartosci i ew. zmniejszenie
		ldi posrednik, 0 ;do porownania
		cpse KROKI,posrednik
		rcall K_MINUS ;zmniejsz
		
		czy_GORA:
		;przycisk S_gora wcisniety?
		sbic PINB,0
		rjmp kroki_END ;jedynka, nie byl wcisniety
		;badanie wartosci i ew. zmniejszenie
		ldi posrednik, 255 ;do porownania
		cpse posrednik, KROKI
		rcall K_PLUS ;zwieksz

		;zakonczono zmiane wartosci KROKow
		
	kroki_END:
	;powrot
	sbis PINB, 2 ;gdy JP2=1, koniec, idz do run
	rjmp SETUP_K
	
	;koniec ustawiania - zapis do EEPROM
	ldi adres, 0 ;adres krokow
	mov dane, KROKI
	rcall ZAPIS_EEPROM
	ldi adres, 1 ;adres temperatury
	mov dane, T_WL
	rcall ZAPIS_EEPROM
	
	rjmp RUN ;czas na prace...
	
;malutkie programiki
T_PLUS:
	inc T_WL
	rcall FILTR
ret
;---
T_MINUS:
	dec T_WL
	rcall FILTR	
ret
;---
K_PLUS:
	inc KROKI
	rcall FILTR ;eliminacja drgan zestykow
ret
;---
K_MINUS:
	dec KROKI
	rcall FILTR ;eliminacja drgan zestykow
ret
;####################################################
;####################################################
RUN: ;normalna praca ukladu
	
	;ustawienie wyjsc silnikow
		ldi posrednik, 0xFF
		out DDRB, posrednik ;wszystko jako wyjscia
		ldi posrednik, 0x00
		out PORTB, posrednik ;stan niski wszedzie (wylacz prad w uzwoj.)
	
	;konfiguracja przetwornika ADC do pracy
		cbi DDRB, 4 ;jako wejscie
		cbi PORTB, 4 ;wylacz pull-up
		ldi posrednik, 0x03 ;ustaw wartosc
			;w diag jest LEWO!!!!
		out ADMUX, posrednik ;wyr. do lewej, kanal PB4
		ldi posrednik, 0x85 ;ustaw wartosc
		out ADCSR, posrednik ;wlacz, preskaler 16

PETLA_RUN:	
	;wykonanie pomiaru
		sbi ADCSR, ADSC ; rozpoczecie pomiaru
		ADC_czekaj:
		sbic ADCSR, ADSC ;oczekuj na zero
		rjmp ADC_czekaj  ;dopoki bedzie 1 - czekaj
		in pomiar, ADCL  ;zapamietaj pomiar
		sbic ADCL, 7     ;czy jest ujemny?
		ldi pomiar, 0x7F ;ustaw max.
		in posrednik, ADCH
		;add pomiar, pomiar ;przetworzenie
	
	;czy pomiar < temp?
		cp pomiar, T_WL ;wykonaj porownanie
		brge licz_DELTA ;nie trzeba wykonywac korekty
		mov pomiar, T_WL ;ustaw dolna wartosc
	
	;wyznaczenie delty
		licz_DELTA:
		mov DELTA, pomiar ;przepisz, aby nie utracic
		sub DELTA, temp ;odejmij od poprzedniego stanu akt. pomiar
		brge DELTA_zero ;okresl, czy wieksza, czy mniejsza
		
		DELTA_mniejsza:
		;err mov DELTA,temp
		neg DELTA ;usun znak
		rcall SILNIK_OBROT_L ;wykanj obrot walu
		rjmp ZWLOKA_WAIT
		
		DELTA_wieksza:
		;err mov DELTA, temp ;przepisz
		rcall SILNIK_OBROT_R ;wykanj obrot walu
		rjmp ZWLOKA_WAIT
			
		DELTA_zero: ;badanie czy DELTA = 0
		ldi posrednik, 0
		cpse DELTA,posrednik
		rjmp DELTA_wieksza
			
	;zwloka czasowa
		ZWLOKA_WAIT:
		ldi wait1, 0xFF ;ustaw
		ldi wait2, 0xFF ;ustaw
		ldi wait3, 0x0F ;ustaw
		ldi posrednik, 0x00 ;do porownan
		ZWLOKA:
		dec wait1
		cpse wait1,posrednik
		rjmp ZWLOKA
		dec wait2
		cpse wait2,posrednik
		rjmp ZWLOKA
		dec wait3
		cpse wait3,posrednik
		rjmp ZWLOKA
		
	mov temp, pomiar ;przepisz i zapamietaj do nast. liczenia
		
	;zapetlenie
rjmp PETLA_RUN
;####################################################
;####################################################
FILTR: ;eliminuje drganie zestykow S_dol i S_gora
	;badanie stany switchy
		sbis PINB,3 ;S_dol
		rjmp FILTR ;jezeli nadal 0 - skocz
		sbis PINB,0 ;S_gora
		rjmp FILTR ;jezeli nadal 0 - skocz
	
	;opoznienie
		ldi wait1, 0xFF ;ustaw
		ldi wait2, 0x18 ;ustaw
		ldi posrednik, 0x00 ;do porownan
		WAIT_F1:
		dec wait1
		cpse wait1,posrednik
		rjmp WAIT_F1
		dec wait2
		cpse wait2,posrednik
		rjmp WAIT_F1
	;-------------------
	
	;badanie stany switchy
		sbis PINB,3 ;S_dol
		rjmp FILTR ;jezeli nadal 0 - skocz
		sbis PINB,0 ;S_gora
		rjmp FILTR ;jezeli nadal 0 - skocz
	
	;opoznienie
		ldi wait1, 0xFF ;ustaw
		ldi wait2, 0x18 ;ustaw
		ldi posrednik, 0x00 ;do porownan
		WAIT_F2:
		dec wait1
		cpse wait1,posrednik
		rjmp WAIT_F2
		dec wait2
		cpse wait2,posrednik
		rjmp WAIT_F2
	;-------------------
	
	;badanie stany switchy
		sbis PINB,3 ;S_dol
		rjmp FILTR ;jezeli nadal 0 - skocz
		sbis PINB,0 ;S_gora
		rjmp FILTR ;jezeli nadal 0 - skocz
ret
;####################################################
;####################################################
SILNIK_OBROT_L:
	;obroty silnika
	mov wait3, KROKI ;ile obr/st.C
	;petla kroki/st.C * DELTA
	
	OBRACAJ_SILNIK_L:
	;realizacja obrotu o krok
	lsl SILNIK ; przesun "napiecie w uzwojeniu"
	;badanie "przekrecenia" licznika ("SILNIK")
	sbrc SILNIK, 4
	ldi SILNIK, 0x01 ;ustaw cykl od poczatku
	;---------
		;przepisanie na wyjscia
		ldi posrednik,0
		out PORTB, posrednik ;wyzeruj wszystko
		sbrc SILNIK, 0
		sbi PORTB,2
		sbrc SILNIK, 1
		sbi PORTB,1
		sbrc SILNIK, 2
		sbi PORTB,3
		sbrc SILNIK, 3
		sbi PORTB,0
	;--------
	;opoznienie
		ldi wait1, 0xFF ;ustaw
		ldi wait2, 0x18 ;ustaw
		ldi posrednik, 0x00 ;do porownan
		WAIT_SL:
		dec wait1
		cpse wait1,posrednik
		rjmp WAIT_SL
		dec wait2
		cpse wait2,posrednik
		rjmp WAIT_SL
	;================================
	ldi posrednik, 0 ;do porownan
	dec wait3
	cpse wait3,posrednik
	rjmp OBRACAJ_SILNIK_L
	mov wait3, KROKI ;ustaw ilosc obrotow
	dec DELTA
	cpse DELTA,posrednik
	rjmp OBRACAJ_SILNIK_L
	out PORTB, posrednik ;wylacz porty
	;*****************************************
ret ;powrot
;####################################################
;####################################################
SILNIK_OBROT_R:
	;obroty silnika
	mov wait3, KROKI ;ile obr/st.C
	;petla kroki/st.C * DELTA
	
	OBRACAJ_SILNIK_R:
	;realizacja obrotu o krok
	lsr SILNIK ; przesun "napiecie w uzwojeniu"
	;badanie "przekrecenia" licznika ("SILNIK")
	ldi posrednik, 0
	cpse SILNIK, posrednik
	rjmp SILNIK_OK
	ldi SILNIK, 0x08 ;jezeli bylo zero - ustaw wart. pocz
	SILNIK_OK:
	;---------
		;przepisanie na wyjscia
		ldi posrednik,0
		out PORTB, posrednik ;wyzeruj wszystko
		sbrc SILNIK, 0
		sbi PORTB,2
		sbrc SILNIK, 1
		sbi PORTB,1
		sbrc SILNIK, 2
		sbi PORTB,3
		sbrc SILNIK, 3
		sbi PORTB,0
	;--------
	;opoznienie
		ldi wait1, 0xFF ;ustaw
		ldi wait2, 0x18 ;ustaw
		ldi posrednik, 0x00 ;do porownan
		WAIT_SR:
		dec wait1
		cpse wait1,posrednik
		rjmp WAIT_SR
		dec wait2
		cpse wait2,posrednik
		rjmp WAIT_SR
	;================================
	ldi posrednik, 0 ;do porownan
	dec wait3
	cpse wait3,posrednik
	rjmp OBRACAJ_SILNIK_R
	mov wait3, KROKI ;ustaw ilosc obrotow
	dec DELTA
	cpse DELTA,posrednik
	rjmp OBRACAJ_SILNIK_R
	out PORTB, posrednik ;wylacz porty
	;*****************************************
ret ;powrot
;####################################################
;####################################################
;obsluga pamieci EEPROM
ZAPIS_EEPROM:
	;ustaw
	out EEAR, adres
	out EEDR, dane
	;zapis
	sbi EECR, EEMWE
	sbi EECR,EEWE
	;oczekiwanie zera
	CZEKAJ_NA_ZERO:
	sbic EECR, EEWE
	rjmp CZEKAJ_NA_ZERO
ret
ODCZYT_EEPROM:
	;ustaw adres
	out EEAR, adres
	;odczyt informacji
	sbi EECR, EERE
	CZEKAJ_ODCZYT:
	sbic EECR, EERE
	rjmp CZEKAJ_ODCZYT
	;odczytaj
	in dane, EEDR
ret